/*******************************************************************************
*  Berzerk.h - Main object to process log files
*
*  Berzerk/
*
*  Part of the BuckoSoft Network Monitor
*  Copyright(c) 2009 - BuckoSoft, Inc.
*
*  http://git.buckosoft.com/gitweb/?p=Berzerk.git;a=history;f=Berzerk/Berzerk.h
*
*  With which we monitor a log file and perform appropriate action.
\*******************************************************************************/

#ifndef	_BERZERK_H_
#define	_BERZERK_H_

#include <string>
#include <fstream>
#include <map>

//#include <boost/regex.hpp>

#include "Filter.h"
#include "Config.h"
#include "LogWriter.h"
#include "Statistics.h"

using namespace std;
using namespace boost;

#define	now()	time(NULL)

///////////////////////////////////////////////////////////////////////////////
class LogFile {
public:
	string		name;
	ifstream	stream;
	int			pos;

};


///////////////////////////////////////////////////////////////////////////////
namespace berzerk {

class BadGuy {
public:
	BadGuy(const string& ip, const string& ruleName, time_t when, int sum);
	string	ip;
	string	ruleName;
	time_t	when;
	int		sum;		// how many badguy points
};
inline 	BadGuy::BadGuy(const string& ip, const string& ruleName, time_t when, int sum) {
	this->ip = ip; this->ruleName = ruleName; this->when = when; this->sum = sum;
}

extern int DEBUG;

class Berzerk {

public:
	Berzerk();
	~Berzerk();

	void	usage();

	int		parseCommandLine(int argc,  _TCHAR* argv[]);
	void	prepareRegexp();
	void	writePidFile();
	int		openLogFiles();
	int		run();
	const Config& getConfig() const;

	void	setTailMode(bool value);	// After reading the file, stick around a la "tail -f"
	bool	isTailMode() const;

	void	addTrap(const string& name, const string& trap);
	void	addFilter(Filter* filter);
	const list<Filter*>	getFilters() const;

	void				 addMappedLogFile(const string& name, const string& filepath);
	const string&		 getMappedLogFile(const string& name);
	const list<LogFile*> getLogFiles();
	const	LogFile*	 getLogFile(int i);

	void	setIpTables(bool value);
	bool	isIpTables() const;

	void	setSeekEnd(bool value);
	bool	isSeekEnd() const;

	void	setSyslog(bool value);
	bool	isSyslog() const;

	void	setDryRun(bool value);
	bool	isDryRun() const;

	void	setDaemonMode(bool value);
	bool	isDaemonMode() const;

	void	setServerAddress(const string& addr);
	const string& getServerAddress() const;

	void	setServerPort(const string& port);
	void	setServerPort(int port);
	const string&	getServerPort() const;

	bool	lineMatched(Filter* filter, const string& line, const regex& rex, const regex& ipPattern);

	Statistics&		getStatistics();

private:

	list<Filter*>	filters;
	list<LogFile*>	logFiles;
	bool			tailMode;
	bool			ipTables;
	bool			bSyslog;
	bool			dryRun;
	bool			seekEnd;
	bool			daemonMode;
	Config			config;
	LogWriter		log;
	map<string,string>	trapMap;
	map<string,string>	logFileMap;
	map<string,BadGuy*>	badguyMap;

	// For the http server
	string			serverAddress;
	string			serverPort;
	Statistics		statistics;
};

inline void Berzerk::setTailMode(bool value) { tailMode = value; };
inline bool	Berzerk::isTailMode() const { return(tailMode); }
inline void Berzerk::setIpTables(bool value) { ipTables = value; };
inline bool	Berzerk::isIpTables() const { return(ipTables); }
inline void Berzerk::setSyslog(bool value) { bSyslog = value; };
inline bool	Berzerk::isSyslog() const { return(bSyslog); }
inline void Berzerk::setSeekEnd(bool value) { seekEnd = value; };
inline bool	Berzerk::isSeekEnd() const { return(seekEnd); }
inline void Berzerk::setDryRun(bool value) { dryRun = value; };
inline bool	Berzerk::isDryRun() const { return(dryRun); }
inline void Berzerk::setDaemonMode(bool value) { daemonMode = value; };
inline bool	Berzerk::isDaemonMode() const { return(daemonMode); }
inline void Berzerk::addFilter(Filter* c) { filters.push_back(c); }
inline void Berzerk::addTrap(const string& n, const string& t) { trapMap[n] = t; }
inline void Berzerk::addMappedLogFile(const string& n, const string& t) { logFileMap[n] = t; }
inline const string& Berzerk::getMappedLogFile(const string& name) { return(logFileMap[name]); }
inline const list<LogFile*> Berzerk::getLogFiles() { return(logFiles); }
inline const list<Filter*>	Berzerk::getFilters() const { return(filters); }
inline void Berzerk::setServerAddress(const string& a) { serverAddress = a; }
inline const string& Berzerk::getServerAddress() const { return(serverAddress); }
inline void Berzerk::setServerPort(const string& s) {serverPort = s; }
inline const string& Berzerk::getServerPort() const { return(serverPort); }
inline Statistics& Berzerk::getStatistics() { return(statistics); }
inline const Config& Berzerk::getConfig() const {return(config); }
extern	Berzerk	berzerk;
}

#endif	// _BERZERK_H_
